package com.example.fgx_super_pay

import androidx.annotation.NonNull

import io.flutter.embedding.engine.plugins.FlutterPlugin
import io.flutter.plugin.common.MethodCall
import io.flutter.plugin.common.MethodChannel
import io.flutter.plugin.common.MethodChannel.MethodCallHandler
import io.flutter.plugin.common.MethodChannel.Result

// import android.app.Activity
// import android.content.ComponentName
// import android.content.Intent
// import android.text.TextUtils
import io.flutter.plugin.common.PluginRegistry
import io.flutter.plugin.common.PluginRegistry.Registrar

/** FgxSuperPayPlugin */
class FgxSuperPayPlugin: FlutterPlugin, MethodCallHandler {
  /// The MethodChannel that will the communication between Flutter and native Android
  ///
  /// This local reference serves to register the plugin with the Flutter Engine and unregister it
  /// when the Flutter Engine is detached from the Activity
  private lateinit var channel : MethodChannel

  override fun onAttachedToEngine(@NonNull flutterPluginBinding: FlutterPlugin.FlutterPluginBinding) {
    channel = MethodChannel(flutterPluginBinding.binaryMessenger, "fgx_super_pay")
    channel.setMethodCallHandler(this)
  }

  private lateinit var deliverResult: Result? = null

  companion object {
      const val price = "pcccc"
      const val period = "poooo"
      const val periodType = "ppppp"
      const val pkgName = "pkgName"
      const val basicUrl = "baseUrl"
      const val pureBasicUrl = "pureBasicUrl"
      const val packageName = "packageName"
      const val serviceName = "serviceName"
      const val errorMsg = "errorMsg"
      const val statusCode = "statusCode"
      const val content = "content"
      const val platform = "platform"

      private lateinit var channel: MethodChannel? = null
      @JvmStatic
      fun registerWith(registrar: Registrar) {
          val plugin = FgxSuperPayPlugin(registrar.activity())
          registrar.addActivityResultListener(plugin)
          channel = MethodChannel(registrar.messenger(), "fgx_super_pay")
          channel!!.setMethodCallHandler(plugin)
      }
  }

  override fun onMethodCall(@NonNull call: MethodCall, @NonNull result: Result) {
    when {
      call.method == "isAppInstalled" -> result.success(isAppInstalled(call.arguments?.toString() ?: ""))
      call.method == "startIntent" -> result.success(startIntent(call))
      call.method == "deliverResult" -> this.deliverResult = result
      else -> result.notImplemented()
    }
  }

  private fun startIntent(call: MethodCall): Boolean {
    val price: String? = call.argument(price)
    val period: Int? = call.argument(period)
    val periodType: String? = call.argument(periodType)
    val pureBasicUrl: String? = call.argument(pureBasicUrl)
    val packageName: String? = call.argument(packageName)
    val serviceName: String? = call.argument(serviceName)

    val intent = Intent().apply {
        putExtra(FgxSuperPayPlugin.price, price)
        period?.let {
            putExtra(FgxSuperPayPlugin.period, period)
        }
        periodType?.let {
            putExtra(FgxSuperPayPlugin.periodType, periodType)
        }
        putExtra(pkgName, activity.packageName)
        putExtra(basicUrl, pureBasicUrl)
        component = ComponentName(packageName!!, serviceName!!)
    }
    activity.startActivityForResult(intent, 4552)
    return true
  }

  private fun isAppInstalled(packageName: String): Boolean {
    if (TextUtils.isEmpty(packageName)) {
        return false
    }

    try {
        activity.packageManager.getPackageInfo(packageName, 0)
        return true
    } catch (e: Exception) {
        return false
    }
  }

  override fun onDetachedFromEngine(@NonNull binding: FlutterPlugin.FlutterPluginBinding) {
    channel.setMethodCallHandler(null)
  }

  override fun onActivityResult(requestCode: Int, resultCode: Int, data: Intent?): Boolean {
    if (requestCode == 4552) {
        if (resultCode == Activity.RESULT_OK) {
            if (data == null) {
                deliverResult?.error("-200", "Response data is null", null)
            } else {
                val err: String? = data.getStringExtra(errorMsg)
                val statusCode: Int? = data.getIntExtra(statusCode, 0)
                val content: String? = data.getStringExtra(content)
                val platform: String? = data.getStringExtra(platform)
                if (err.isNullOrEmpty()) {
                    deliverResult?.success(mapOf(
                            "code" to statusCode,
                            "content" to content,
                            "platform" to platform
                    ))
                } else {
                    if (statusCode == 1) {
                        deliverResult?.error("-201", "Purchase Cancelled", null)
                    } else {
                        deliverResult?.error("-100", "response err $err---$platform", null)
                    }
                }
            }
        } else if (resultCode == Activity.RESULT_CANCELED) {
            deliverResult?.error("-201", "Purchase Cancelled", null)
        }
        return true
    }
    return false
  }
}
