// ignore_for_file: prefer_typing_uninitialized_variables

import 'package:dio/dio.dart';

import 'dio_call.dart';
import 'dio_http.dart';
import 'dio_request.dart';

///
/// author：Vans Z
/// date： 2019/1/13 0013
///

abstract class RequestBuilder {
  DioHttp dioHttp;
  String url;
  String? tag;
  Map<String, dynamic>? headerMap;
  dynamic params;
  Options? options;

  RequestBuilder(this.dioHttp, this.url,
      {this.tag, this.headerMap, this.params, this.options});

  RequestBuilder setHeader(Map<String, dynamic> header) {
    headerMap = header;
    return this;
  }

  RequestBuilder setParams(dynamic params) {
    this.params = params;
    return this;
  }

  RequestBuilder addHeader(String key, var value) {
    headerMap ??= {};
    headerMap?[key] = value;
    return this;
  }

  RequestBuilder addParam(String key, var value) {
    params ??= {};
    params[key] = value;
    return this;
  }

  DioCall build();
}

class GetBuilder extends RequestBuilder {
  GetBuilder(DioHttp dioHttp, String? url)
      : assert(url != null),
        super(dioHttp, url!);

  @override
  DioCall build() {
    return GetRequest(
      dioHttp,
      url,
      tag: tag,
      headerMap: headerMap,
      paramMap: params,
      options: options,
    ).build();
  }
}

class PostBuilder extends RequestBuilder {
  PostBuilder(DioHttp dioHttp, String? url)
      : assert(url != null),
        super(dioHttp, url!);

  @override
  DioCall build() {
    return PostRequest(
      dioHttp,
      url,
      tag: tag,
      headerMap: headerMap,
      paramMap: params,
      options: options,
    ).build();
  }
}

class FormBuilder extends RequestBuilder {
  FormBuilder(
    DioHttp dioHttp,
    String? url,
    Map<String, dynamic>? paramMap,
  )   : assert(url != null),
        super(dioHttp, url!, params: paramMap);


  @override
  RequestBuilder setParams(params) {
    // merge params
    this.params ??= <String, dynamic>{};
    if (params is Map<String, dynamic>) {
      params.forEach((key, value){
        this.params[key] = value;
      });
    }
    return this;
  }

  @override
  DioCall build() {
    return FormRequest(
      dioHttp,
      url,
      tag: tag,
      headerMap: headerMap,
      paramMap: params,
      options: options,
    ).build();
  }
}

class DownloadBuilder extends RequestBuilder {
  final String? savePath;
  var progress;

  DownloadBuilder(DioHttp dioHttp, String? url, this.savePath)
      : assert(url != null && url.isNotEmpty),
        assert(savePath != null && savePath.isNotEmpty),
        super(dioHttp, url!);

  @override
  DioCall build() {
    return DownloadRequest(
      dioHttp,
      url,
      savePath,
      progress: progress,
      paramMap: params,
      headerMap: headerMap,
      options: options,
    ).build();
  }
}
