// ignore_for_file: prefer_final_fields

import 'package:dio/dio.dart';
import 'package:dio/adapter.dart';
import 'dio_builder.dart';

///
/// author：Vans Z
/// date： 2019/1/13 0013
/// desc: 提供 http 通信功能，可以被其他类 mixin
///

class DioHttp {
  static const String defaultTag = 'dioHttpTag';

  Dio _dio = Dio();

  Map<String, CancelToken> tokenMap = {};
  late Map<String, dynamic>? commonParams;

  Dio getDioClient() => _dio;

  void init(BaseOptions options, {required Map<String, dynamic> commonParam, required String? proxy}) {
    _dio.options = options;
    commonParams = commonParam;

    _dio.interceptors.add(LogInterceptor(responseBody: false));

    /// proxy
    bool isProxyChecked = proxy != null && proxy.isNotEmpty;
    (_dio.httpClientAdapter as DefaultHttpClientAdapter).onHttpClientCreate = (client) {
      client.findProxy = (uri) {
        return isProxyChecked ? 'PROXY $proxy' : 'DIRECT';
      };
    };
  }

  /// add  interceptor
  void setInterceptor({
    required InterceptorSendCallback onRequest,
    required InterceptorSuccessCallback onResponse,
    required InterceptorErrorCallback onError,
  }) {
    _dio.interceptors.add(InterceptorsWrapper(
      onRequest: onRequest,
      onResponse: onResponse,
      onError: onError,
    ));
  }

  /// add or remove Transformer
  void setTransformer(Transformer transformer) {
    _dio.transformer = transformer;
  }

  GetBuilder get(String url) => GetBuilder(this, url);

  PostBuilder post(String url) => PostBuilder(this, url);

  FormBuilder form(String url, {Map<String, dynamic>? formData}) => FormBuilder(this, url, formData);

  DownloadBuilder download(String url, String savePath) => DownloadBuilder(this, url, savePath);

  void cancel({String? tag, String? message}) {
    var cancelToken = tokenMap[tag ?? defaultTag];
    if (cancelToken != null) cancelToken.cancel(message);
  }

  testNetwork() async => await get('http://www.baidu.com').build().execute();
}
