// ignore_for_file: prefer_typing_uninitialized_variables

import 'dart:async';

import 'package:dio/dio.dart';
import 'package:flutter/foundation.dart';

import 'dio_call.dart';
import 'dio_http.dart';

///
/// author：Vans Z
/// date： 2019/1/13 0013
///

abstract class DioRequest {
  late DioHttp dioHttp;
  late String url;
  late String? tag;
  late dynamic params;
  late Map<String, dynamic>? headerMap;
  late Options? options;

  late CancelToken cancelToken;

  DioRequest(
    this.dioHttp,
    this.url, {
    this.tag,
    this.params,
    this.headerMap,
    this.options,
  }) {
    cancelToken = CancelToken();
  }

  DioCall build() {
    return DioCall(this);
  }

  Future<Response> callRequest(Dio dio);

  void log() {
    debugPrint('DioHttp      url: $url');
    debugPrint('DioHttp      tag: $tag');
    debugPrint('DioHttp   params: $params');
    debugPrint('DioHttp  headers: $headerMap');
  }

  @override
  String toString() {
    return 'DioRequest{'
        'url: $url,'
        'tag: $tag,'
        'paramMap: $params,'
        'headerMap: $headerMap,'
        'cancelToken: $cancelToken}';
  }
}

class GetRequest extends DioRequest {
  GetRequest(
    DioHttp dio,
    String url, {
    id,
    tag,
    paramMap,
    headerMap,
    options,
  }) : super(
          dio,
          url,
          tag: tag,
          params: paramMap,
          headerMap: headerMap,
          options: options,
        );

  @override
  Future<Response> callRequest(Dio dio) {
    return dio.get(
      url,
      queryParameters: params,
      cancelToken: cancelToken,
      options: options,
    );
  }
}

class PostRequest extends DioRequest {
  PostRequest(
    DioHttp dio,
    String url, {
    id,
    tag,
    paramMap,
    headerMap,
    options,
  }) : super(
          dio,
          url,
          tag: tag,
          params: paramMap,
          headerMap: headerMap,
          options: options,
        );

  @override
  Future<Response> callRequest(Dio dio) {
    return dio.post(
      url,
      data: params,
      cancelToken: cancelToken,
      options: options,
    );
  }
}

class FormRequest extends DioRequest {
  FormRequest(
    DioHttp dioHttp,
    String url, {
    tag,
    paramMap,
    headerMap,
    options,
  }) : super(
          dioHttp,
          url,
          tag: tag,
          params: paramMap,
          headerMap: headerMap,
          options: options,
        );

  @override
  Future<Response> callRequest(Dio dio) {
    FormData formData = FormData.fromMap(params);
    return dio.post(
      url,
      data: formData,
      cancelToken: cancelToken,
      options: options,
    );
  }
}

class DownloadRequest extends DioRequest {
  late final String? savePath;
  late var progress;

  DownloadRequest(
    DioHttp dioHttp,
    String url,
    this.savePath, {
    progress,
    tag,
    paramMap,
    headerMap,
    options,
  }) : super(
          dioHttp,
          url,
          tag: tag,
          params: paramMap,
          headerMap: headerMap,
          options: options,
        );

  @override
  Future<Response> callRequest(Dio dio) {
    return dio.download(
      url,
      savePath,
      onReceiveProgress: progress,
      data: params,
      cancelToken: cancelToken,
      options: options,
    );
  }

  @override
  void log() {
    debugPrint('DioHttp savePath: $url');
    super.log();
  }
}
