import 'dart:async';
import 'dart:convert';

import 'package:shared_preferences/shared_preferences.dart';

///
/// date： 2019/06/03
/// desc: 持久化各种类型数据到文件当中
/// warn: 存储对象类型数据，需要对象类中重写 toJson 方法
///
class LocalStorage {
  LocalStorage._internal();

  static LocalStorage _singleton = LocalStorage._internal();

  factory LocalStorage() => _singleton;

  _getPrefs() => SharedPreferences.getInstance();

  String _obj2Str(data) {
    if (data is! String) {
      return jsonEncode(data);
    }
    return data.toString();
  }

  dynamic _data2Obj(data) {
    if (data is! int &&
        data is! double &&
        data is! bool &&
        data is! String &&
        data is! List) {
      return jsonDecode(data);
    } else {
      return data;
    }
  }

  Future<dynamic> getItem(String key) async {
    var data = (await _getPrefs()).get(key);
    return data == null ? null : _data2Obj(data);
  }

  Future<bool> setItem(String key, value) async {
    SharedPreferences prefs = await _getPrefs();
    Future<bool> future;
    if (value is int) {
      future = prefs.setInt(key, value);
    } else if (value is double) {
      future = prefs.setDouble(key, value);
    } else if (value is bool) {
      future = prefs.setBool(key, value);
    } else if (value is List<String>) {
      future = prefs.setStringList(key, value);
    } else {
      future = prefs.setString(key, _obj2Str(value));
    }
    return future;
  }

  Future<bool> removeItem(String key) async => (await _getPrefs()).remove(key);

  Future<dynamic> getItemByUser(String userId, String key) async {
    Map data = await getItem(key);
    return data == null ? null : data[userId];
  }

  Future<bool> setItemByUser(String userId, String key, value) async {
    Map data = (await getItem(key)) ?? {};
    data[userId] = _obj2Str(value);
    return setItem(key, data);
  }

  Future<bool> removeItemByUser(String userId, String key) async {
    Map data = (await getItem(key)) ?? {};
    data.remove(userId);
    return setItem(key, data);
  }
}

final localStorage = LocalStorage();
