// ignore_for_file: prefer_typing_uninitialized_variables

import 'package:fgx_super_pay/fgx_super_pay.dart';
import 'package:flutter/services.dart';

///
/// author：Vans Z
/// date： 2019-09-16
///
class PayPlugin {
  PayPlugin._internal();

  static final PayPlugin _singleton = PayPlugin._internal();

  factory PayPlugin() => _singleton;

  static const MethodChannel _channel = MethodChannel('fgx_super_pay');

  static const onPaySuccess = "on_pay_success";
  static const onPayFailed = "on_pay_failed";

  Future<bool> isAppInstalled(String packageName) async {
    var install;
    try {
      install = await _channel.invokeMethod('isAppInstalled', packageName);
    } catch (_) {
      install = false;
    }
    return install;
  }

  Future<bool> startActivityForResult(Map<String, dynamic> params) async {
    var start;
    try {
      start = await _channel.invokeMethod("startIntent", params);
    } catch (_) {
      start = false;
    }
    return start;
  }

  /// 接受 super pay 的支付结果
  Future<PayResult> deliverResult() async {
    try {
      var result = await _channel.invokeMethod("deliverResult");
      return PayResult.success(result);
    } on PlatformException catch (e) {
      return PayResult.failed(code: e.code, errMessage: e.message);
    }
  }
}

final plugin = PayPlugin();
